unit hiAdjustControls;

interface

uses Windows, Messages, Kol, Share, Debug, Win, hiMTStrTbl, hiStringTable, hiStringTableMT;

const
  DTM_FIRST            = $1000;
  DTM_GETSYSTEMTIME    = DTM_FIRST + 1;
  DTM_SETSYSTEMTIME    = DTM_FIRST + 2;
  GDT_VALID            = 0;
  
  CLEAR_FULL           = 0;
  CLEAR_COLUMNS        = 1;
  CLEAR_TABLE          = 2;

  AlignName: array [0..5] of string = (
    'none','left','top','rigth','bottom','client' );
     
  PropNames: array[0..29] of string = (
    'left','top','width','height','align','color',
    'text','caption','strings','visible','enabled','checked','selected',
    'select','fontname','fontcolor','fontsize', 'fontstyle','fontcharset',
    'redraw', 'sendtoback', 'bringtofront', 'setfocus', 'clear',
    'columns', 'table', 'tabs', 'colors', 'datetime', 'edit' );

  ColorValues: array [0..46] of TColor = (
    clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray,
    clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite,
    clMoneyGreen,clSkyBlue,clCream,clMedGray,clActiveBorder,clActiveCaption,
    clAppWorkSpace,clBackground,clBtnFace,clBtnHighlight,clBtnShadow,    
    clBtnText,clCaptionText,clDefault,clGrayText,clHighlight,clHighlightText,
    clInactiveBorder,clInactiveCaption,clInactiveCaptionText,clInfoBk,
    clInfoText,clMenu,clMenuText,clNone,clScrollBar,cl3DDkShadow,
    cl3DLight,clWindow,clWindowFrame,clWindowText );

  ColorNames: array [0..46] of string = (
    'black','maroon','green','olive','navy','purple','teal','gray',
    'silver','red','lime','yellow','blue','fuchsia','aqua','white',
    'moneygreen','skyblue','cream','medgray','activeborder','activecaption',
    'appworkspace','background','btnface','btnhighlight','btnshadow',    
    'btntext','captiontext','default','graytext','highlight','highlighttext',
    'inactiveborder','inactivecaption','inactivecaptiontext','infobk',
    'infotext','menu','menutext','none','scrollbar','3ddkshadow',
    '3dlight','window','windowframe','windowtext' );

type
  THIAdjustControls = class(TDebug)
   private
    FListSet,
    FListIncludeName,
    FListExcludeName,
    FListIncludeProp,
    FListExcludeProp,
    FListProp: PStrList;

    procedure SetParameters(const Value: string);
    procedure SetIncludeNameList(const Value: string);
    procedure SetExcludeNameList(const Value: string);
    procedure SetIncludePropList(const Value: string);
    procedure SetExcludePropList(const Value: string);
    function GetIntColor(const Value: string): integer;    
    function GetStrColor(const Value: integer): string;
    function isTrueName(Name: string): boolean;
    function isTrueProp(Prop: string): boolean;
   public
    _prop_NameControl,
    _prop_Property,
    _prop_Value: byte;
    
    _prop_FormatDate,
    _prop_Parameters,
    _prop_IncludeNameList,
    _prop_ExcludeNameList,
    _prop_IncludePropList,
    _prop_ExcludePropList: string;
    _data_NameControl,
    _data_Parameters,
    _data_IncludeNameList,
    _data_ExcludeNameList,
    _data_IncludePropList,
    _data_ExcludePropList,
    _event_onSet,
    _event_onEnumNameControls,
    _event_onEndEnumNameControls,
    _event_onEnumParameters,
    _event_onEndEnumParameters: THI_Event;
    
    constructor Create;
    destructor Destroy; override;
    procedure _work_doSet(var _Data: TData; Index: word);
    procedure _work_doEnumNameControls(var _Data: TData; Index: word);
    procedure _work_doEnumParameters(var _Data: TData; Index: word);
  end;
    
implementation

uses hiStr_Enum;

constructor THIAdjustControls.Create;
var
  i: integer;
begin
   inherited Create;
   FListSet := NewStrList;
   FListProp := NewStrList;
   FListIncludeName := NewStrList;
   FListExcludeName := NewStrList; 
   FListIncludeProp := NewStrList;
   FListExcludeProp := NewStrList;
   for i := 0 to High(PropNames) do 
     FListProp.Add(PropNames[i]);
end;

destructor THIAdjustControls.Destroy;
begin
   FListSet.Free;
   FListProp.Free;
   FListIncludeName.Free;
   FListExcludeName.Free;
   FListIncludeProp.Free;
   FListExcludeProp.Free;
   inherited;
end;

function THIAdjustControls.GetIntColor;
var
  i: integer;
begin
  for i := 0 to High(ColorNames) do
    if Value = ColorNames[i] then
    begin
      Result := ColorValues[i];
      exit;
    end;
  Result := str2int(Value);
end;

function THIAdjustControls.GetStrColor;
var
  i: integer;
begin
  for i := 0 to High(ColorValues) do
    if Color2RGB(Value) = Color2RGB(ColorValues[i]) then
    begin
      Result := ColorNames[i];
      exit;
    end;
  Result := int2str(Value);
end;

function THIAdjustControls.isTrueName(Name: string): boolean;
begin
  Result := ((FListIncludeName.IndexOf(Name) <> -1) or
            (FListIncludeName.IndexOf('all') <> -1)) and
            (FListExcludeName.IndexOf(Name) = -1);
end;

function THIAdjustControls.isTrueProp(Prop: string): boolean;
begin
  Result := ((FListIncludeProp.IndexOf(prop) <> -1) or
            (FListIncludeProp.IndexOf('all') <> -1)) and
            (FListExcludeProp.IndexOf(prop) = -1);
end;

//------------------------------- Set Lists ------------------------------------

procedure THIAdjustControls.SetParameters;
begin
  if Value <> '' then FListSet.SetText(Value, false)
end;

procedure THIAdjustControls.SetIncludeNameList;
begin
  if Value <> '' then FListIncludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludeNameList;
begin
  if Value <> '' then FListExcludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetIncludePropList;
begin
  if Value <> '' then FListIncludeProp.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludePropList;
begin
  if Value <> '' then FListExcludeProp.SetText(LowerCase(Value), false);
end;

//------------------------------------------------------------------------------

procedure THIAdjustControls._work_doSet;
var
  tmpControl: PControl;
  i, j, val, _itc: integer;
  dt: TDateTime;
  st: TSystemTime;
  _str, str, _ntc, s_name, _s_name, s_prop: string;
  Mode: Cardinal;
  aName: array[0..255] of Char;
  
   procedure change(ind: integer);
   var
     k: integer;
   begin
     GetClassName(tmpControl.Handle, aName, 256);
//     _debug(aName + ' - ' + tmpControl.SubClassName); 
     with tmpControl{$ifndef F_P}^{$endif} do
     case ind of
       0: Left := str2int(str);
       1: Top := str2int(str);
       2: Width := str2int(str);
       3: Height := str2int(str);
       4: begin
            if (LowerCase(str) = AlignName[0]) or (str ='0') then Align := caNone   else // none
            if (LowerCase(str) = AlignName[1]) or (str ='1') then Align := caLeft   else // left
            if (LowerCase(str) = AlignName[2]) or (str ='2') then Align := caTop    else // top         
            if (LowerCase(str) = AlignName[3]) or (str ='3') then Align := caRight  else // rigth 
            if (LowerCase(str) = AlignName[4]) or (str ='4') then Align := caBottom else // bottom 
            if (LowerCase(str) = AlignName[5]) or (str ='5') then Align := caClient;     // client 
            Invalidate;
          end;
       5: Color := GetIntColor(LowerCase(str)); 
       6: if SubClassName = 'obj_EditControl' then Text := str;  // text

       7: if isForm then // caption
            SetWindowText(Handle, @str[1])
          else if (SubClassName = 'obj_BUTTON') or
                  (SubClassName = 'obj_STATIC')  then
          begin 
            Replace(str, '|', #13#10);
            Text := str;
          end;  

       8: if (SubClassName = 'obj_LISTBOX') or // strings
             (SubClassName = 'obj_COMBOBOX') then
          begin
            Clear;
            while str <> '' do
            begin
              _str := fparse(str,'|');
              Add(fparse(_str,'#'));
              ItemData[Count - 1] := str2int(_str); 
            end;
          end
          else if (SubClassName = 'obj_MemoControl') or
                  (SubClassName = 'obj_RichEdit20A') then
          begin 
            Replace(str, '|', #13#10);
            Text := str;
          end;
   
       9: Visible := boolean(ord(str2int(str)));
      10: Enabled := boolean(ord(str2int(str)));
   11,12: Checked := boolean(ord(str2int(str))); // checked = selected
      13: begin // select -  
            val := str2int(str);
            if (val < 0) or (val >= Count) then val := -1;
            if not ((val < 0) and (aName = 'obj_COMBOBOX')) then
              CurIndex := val;
          end;
      14: begin 
            Font.FontName := str;
            if (aName = 'obj_SysListView32') then InvalidateEx;
          end;  
      15: begin
            Font.Color := GetIntColor(LowerCase(str));
            if (aName = 'obj_SysListView32') then
            begin
              LVTextColor := GetIntColor(LowerCase(str));
              InvalidateEx;
            end;
          end;
      16: begin
            Font.FontHeight := _hi_SizeFnt(str2int(str));
            if (aName = 'obj_SysListView32') then InvalidateEx;
          end;  
      17: begin
            Font.FontStyle := [];
            _str := LowerCase(str);
            for k := 1 to length(_str) do
            begin
              if _str[k] = 'b' then Font.FontStyle := Font.FontStyle + [fsBold] else
              if _str[k] = 'i' then Font.FontStyle := Font.FontStyle + [fsItalic] else
              if _str[k] = 'u' then Font.FontStyle := Font.FontStyle + [fsUnderline] else
              if _str[k] = 's' then Font.FontStyle := Font.FontStyle + [fsStrikeOut];
            end;
            if (aName = 'obj_SysListView32') then InvalidateEx;
          end;
      18: begin
            Font.FontCharset := str2int(str);
            if (aName = 'obj_SysListView32') then InvalidateEx;
          end;  
      19: if str = 'true' then
              InvalidateRect(Handle, nil, true)
            else
              InvalidateRect(Handle, nil, false);
      20: tmpControl.SendToBack;
      21: tmpControl.BringToFront;
      22: Focused := true;
      
      23: begin
            if str = 'columns' then Mode := CLEAR_COLUMNS
            else if str = 'table' then Mode := CLEAR_TABLE
            else Mode := CLEAR_FULL;  
            if SubClassName = 'obj_MTStrTbl' then
                ThiMTStrTbl(Tag).FullClear(Mode)
            else if SubClassName = 'obj_StringTable' then
                ThiStringTable(Tag).FullClear(Mode)
            else if SubClassName = 'obj_StringTableMT' then
                ThiStringTableMT(Tag).FullClear(Mode)
            else     
              Clear;
          end;
          
      24: if SubClassName = 'obj_MTStrTbl' then
            ThiMTStrTbl(Tag).FullLoadColumns(str, '|')
          else if SubClassName = 'obj_StringTable' then
            ThiStringTable(Tag).FullLoadColumns(str, '|')
          else if SubClassName = 'obj_StringTableMT' then
              ThiStringTableMT(Tag).FullLoadColumns(str, '|');

      25: if SubClassName = 'obj_MTStrTbl' then
            ThiMTStrTbl(Tag).FullLoadTable(str, ';', '|')
          else if SubClassName = 'obj_StringTable' then
            ThiStringTable(Tag).FullLoadTable(str, ';', '|')       
          else if SubClassName = 'obj_StringTableMT' then
            ThiStringTableMT(Tag).FullLoadTable(str, ';', '|');       

      26: if (SubClassName = 'obj_SysTabControl32') then // tabs
          begin
            for k := Count - 1 downto 0 do TC_Delete(k);
            while str <> '' do
            begin
              _str := fparse(str,'|');
              _ntc := fparse(_str,'#');
              if _str = '' then
                _itc := Count
              else
                _itc := str2int(_str);
              TC_Insert(Count, _ntc, _itc);
            end;  
          end;

      27: if (SubClassName = 'obj_ColorBox') then                  // colors
          begin
              Clear;
              while str <> '' do
              begin
                _str := fparse(str,'|');
                Add(fparse(_str,'#'));
                ItemData[Count - 1] := str2int(_str); 
              end;
            end;

      28: if (SubClassName = 'obj_SysDateTimePick32') then
          begin
            if  length(str) > 0 then
            begin
              if (str[1] = '@') then
              begin
                Replace(str, '@', '0');
                dt := str2double(str); 
              end
              else
                dt := Str2DateTimeFmt(_prop_FormatDate, str);              
              DateTime2SystemTime(dt, st);
            end
            else
              GetLocalTime(st);  
            Perform(DTM_SETSYSTEMTIME, GDT_VALID, Longint(@st));
          end;

      29: if SubClassName = 'obj_COMBOBOX' then Text := str; 
     end;
   end;
   

begin
  SetParameters(ReadString(_Data, _data_Parameters, _prop_Parameters));
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  for i := 0 to ListOfNameControls.Count - 1 do
  begin
    tmpControl := PControl(ListOfNameControls.Objects[i]);
    with tmpControl{$ifndef F_P}^{$endif}  do
    if isControl or isForm then
    begin
      s_name := '';
      for j := 0 to FListSet.Count - 1 do
      begin
        str := FListSet.Items[j];
        if str <> '' then
        begin
          _s_name := LowerCase(fparse(str,'='));
          if (_s_name <> s_name) and (_s_name <> '*') then s_name := _s_name; 
          if ((s_name = LowerCase(ListOfNameControls.Items[i])) or (s_name = 'all')) and isTrueName(s_name) then
          begin
            s_prop := LowerCase(fparse(str,'='));
            if isTrueProp(s_prop) then change(FListProp.IndexOf(s_prop));
          end;
        end;
      end;
    end;
  end;
  _hi_OnEvent(_event_onSet);
end;

procedure THIAdjustControls._work_doEnumNameControls;
var
  i: integer;
begin
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));

  for i := 0 to ListOfNameControls.Count - 1 do
    if isTrueName(ListOfNameControls.Items[i]) then
      _hi_onEvent(_event_onEnumNameControls, ListOfNameControls.Items[i]);
  _hi_onEvent(_event_onEndEnumNameControls);      
end;

procedure THIAdjustControls._work_doEnumParameters;
var
  r: real;
  i: integer;
  dt: TDateTime;
  st: TSystemTime;  
  sFontSize: integer;
  tmpControl: PControl;
  name, sFontStyle, sText: string;
//  dtname, dtprop, dtval: TData;
  dp: TData;
  mt: PMT;  
//  aName: array[0..255] of Char;  
  
  procedure outparameters(prop, value: string);
  begin
    if isTrueProp(prop) then
    begin   
      dtNull(dp);
      if _prop_NameControl = 0 then
        dtString(dp, name)
      else if _prop_Property = 0 then   
        dtString(dp, prop)
      else if _prop_Value = 0 then
        dtString(dp, value);
      mt := mt_make(dp);
      if _prop_NameControl = 1 then
        mt_string(mt, name)
      else if _prop_Property = 1 then   
        mt_string(mt, prop)
      else if _prop_Value = 1 then
        mt_string(mt, value);
      if _prop_NameControl = 2 then
        mt_string(mt, name)
      else if _prop_Property = 2 then   
        mt_string(mt, prop)
      else if _prop_Value = 2 then
        mt_string(mt, value);
//      dtval.ldata := @dtprop; 
//      _hi_onEvent_(_event_onEnumParameters, dtname);
      _hi_onEvent_(_event_onEnumParameters, dp);
      mt_free(mt);
    end;  
  end;

begin
  name := LowerCase(ReadString(_Data, _data_NameControl));
  if name = '' then exit;

  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  if not isTrueName(name) then exit;
  tmpControl := PControl(ListOfNameControls.Objects[ListOfNameControls.IndexOf(name)]);
  if not Assigned(tmpControl) then exit;
  
  if not (tmpControl.isControl or tmpControl.isForm) then exit;

//  dtString(dtname, name); 
//  dtname.ldata := @dtval;

  with tmpControl{$ifndef F_P}^{$endif} do
  begin
//    GetClassName(Handle, aName, 256);
    outparameters(PropNames[0], int2str(Left));              // left
    outparameters(PropNames[1], int2str(Top));               // top
    outparameters(PropNames[2], int2str(Width));             // width
    outparameters(PropNames[3], int2str(Height));            // height
    outparameters(PropNames[4], AlignName[ord(Align)]);      // align
    outparameters(PropNames[5], GetStrColor(Color));         // color

    if SubClassName = 'obj_EditControl' then                 // text  
    begin
      sText := Text;
      outparameters(PropNames[6], sText);    
    end;
    
    if isForm then                                           // caption
    begin
      sText := Text;
      outparameters(PropNames[7], sText);
    end  
    else if (SubClassName = 'obj_BUTTON') or
            (SubClassName = 'obj_STATIC')  then
    begin 
      sText := Text;
      Replace(sText, #13#10, '|');
      outparameters(PropNames[7], sText);
    end; 

    if (SubClassName = 'obj_LISTBOX') or                     // strings
       (SubClassName = 'obj_COMBOBOX') then
    begin
      sText := '';
      for i := 0 to Count - 1 do 
        sText := sText + Items[i] + '#' + int2str(ItemData[i]) + '|';
      outparameters(PropNames[8], sText);
    end
    else if (SubClassName = 'obj_MemoControl') or
            (SubClassName = 'obj_RichEdit20A') then
    begin 
      sText := Text;
      Replace(sText, #13#10, '|');
      outparameters(PropNames[8], sText);
    end; 

    outparameters(PropNames[10], int2str(ord(Enabled)));     // enabled
    outparameters(PropNames[11], int2str(ord(Checked)));     // checked                    
    outparameters(PropNames[12], int2str(ord(Checked)));     // selected
   
    outparameters(PropNames[14], Font.FontName);             // fontname  
    outparameters(PropNames[15], GetStrColor(Font.Color));   // fontcolor
    r := ((Font.FontHeight * -72) - 36) / ScreenDPI;
    sFontSize := Trunc(r);
    if Frac(r) > 0 then Inc(sFontSize);
    outparameters(PropNames[16], int2str(sFontSize));        // fontsize
    sFontStyle := '';
    if fsBold in Font.FontStyle then  
      sFontStyle := sFontStyle + 'b';
    if fsItalic in Font.FontStyle then
      sFontStyle := sFontStyle + 'i';     
    if fsUnderline in Font.FontStyle then
      sFontStyle := sFontStyle + 'u';     
    if fsStrikeOut in Font.FontStyle then
      sFontStyle := sFontStyle + 's';     
    outparameters(PropNames[17], sFontStyle);                // fontstyle     
    outparameters(PropNames[18], int2str(Font.FontCharset)); // fontcharset

    if SubClassName = 'obj_MTStrTbl' then
    begin
      sText := ThiMTStrTbl(Tag).FullSaveColumns('|');
      outparameters(PropNames[24], sText);                   // columns
    end
    else if SubClassName = 'obj_StringTable' then
    begin
      sText := ThiStringTable(Tag).FullSaveColumns('|');
      outparameters(PropNames[24], sText);                   // columns
    end  
    else if SubClassName = 'obj_StringTableMT' then
    begin
      sText := ThiStringTableMT(Tag).FullSaveColumns('|');
      outparameters(PropNames[24], sText);                   // columns
    end; 
    
    if SubClassName = 'obj_MTStrTbl' then
    begin
      sText := ThiMTStrTbl(Tag).FullSaveTable(';', '|');    
      outparameters(PropNames[25], sText);                   // table
    end
    else if SubClassName = 'obj_StringTable' then
    begin
      sText := ThiStringTable(Tag).FullSaveTable(';', '|');    
      outparameters(PropNames[25], sText);                   // table
    end
    else if SubClassName = 'obj_StringTableMT' then
    begin
      sText := ThiStringTableMT(Tag).FullSaveTable(';', '|');    
      outparameters(PropNames[25], sText);                   // table
    end;
    
    if SubClassName = 'obj_SysTabControl32' then
    begin
      sText := '';
      for i := 0 to Count - 1 do 
        sText := sText + TC_Items[i] + '#' + int2str(TC_Images[i]) + '|';
      outparameters(PropNames[26], sText);                   // tabs        
    end;

    if (SubClassName = 'obj_ColorBox') then                  // colors
    begin
      sText := '';
      for i := 0 to Count - 1 do 
        sText := sText + Items[i] + '#' + int2str(ItemData[i]) + '|';
      outparameters(PropNames[27], sText);
    end;
    
    if SubClassName = 'obj_SysDateTimePick32' then
    begin
      Perform(DTM_GETSYSTEMTIME, 0, Longint(@st));
      SystemTime2DateTime(st, dt);
      sText := '@' + double2str(dt);
      outparameters(PropNames[28], sText);                   // datetime
    end;

    if SubClassName = 'obj_COMBOBOX' then
    begin
      sText := Text;
      outparameters(PropNames[29], sText);                   // edit
    end;  

    outparameters(PropNames[13], int2str(CurIndex));         // select
    outparameters(PropNames[9], int2str(ord(Visible)));      // visible
         
  end;
  _hi_onEvent(_event_onEndEnumParameters);   
end;

end.